// 
//  Copyright 2011  campbelk
// 
//    Licensed under the Apache License, Version 2.0 (the "License");
//    you may not use this file except in compliance with the License.
//    You may obtain a copy of the License at
// 
//        http://www.apache.org/licenses/LICENSE-2.0
// 
//    Unless required by applicable law or agreed to in writing, software
//    distributed under the License is distributed on an "AS IS" BASIS,
//    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//    See the License for the specific language governing permissions and
//    limitations under the License.

using System;
using System.Collections.Generic;
using Mono.Unix;
using ocmengine;

namespace ocmgtk
{


	public partial class FieldNotesProgress : Gtk.Dialog
	{
		UIMonitor m_mon;
		bool m_cancel = false;

		public FieldNotesProgress ()
		{
			this.Build ();
			m_mon = UIMonitor.getInstance();
		}
		
		public DateTime ProcessFieldNotes(GPSProfile profile, DateTime latestScan, String ownerId)
		{
					statusLabel.Markup = Catalog.GetString("<i>Receiving from device...</i>");
					UpdateNow();
					List<CacheLog> logs = FieldNotesHandler.GetLogs(profile.FieldNotesFile, ownerId);
					int iCount = 0;
					int iProgCount = 0;
					int iTotal = logs.Count;
					statusLabel.Markup = Catalog.GetString("<i>Processing Field Notes...</i>");
					foreach(CacheLog log in logs)
					{
						if (m_cancel)
							return DateTime.MinValue;
						double prog = (double)((double) iProgCount/ (double) iTotal);
						loadProgress.Fraction = prog;
						loadProgress.Text = prog.ToString("0%");
						iProgCount ++;
						UpdateNow();
						if (log.LogDate <= latestScan)
							continue;
						Geocache cache = Engine.getInstance().Store.GetCache(log.CacheCode);
						m_mon.ProcessOfflineLog(cache, log, false);
						if (latestScan < log.LogDate)
							latestScan = log.LogDate;
						iCount ++;
					}
					buttonCancel.Visible = false;
					buttonClose.Visible = true;
					buttonView.Visible = true;
					loadProgress.Fraction = 1;
					statusLabel.Markup = String.Format(Catalog.GetString("<i>Scanned {0} Field Notes, {1} new.</i>"), iProgCount, iCount);
					loadProgress.Text = Catalog.GetString("Complete");
					return latestScan;
		}
		
		private void UpdateNow()
		{
			while (Gtk.Application.EventsPending ())
				Gtk.Application.RunIteration (false);
		}
		protected virtual void OnViewClick (object sender, System.EventArgs e)
		{
			this.Hide();
			m_mon.ViewOfflineLogs();
		}
		
		protected virtual void OnCloseClick (object sender, System.EventArgs e)
		{
			this.Hide();
		}
		
		protected virtual void OnCancelClick (object sender, System.EventArgs e)
		{
			this.Hide();
		}
		
		
		
		
	}
}
