// 
//  Copyright 2011  campbelk
// 
//    Licensed under the Apache License, Version 2.0 (the "License");
//    you may not use this file except in compliance with the License.
//    You may obtain a copy of the License at
// 
//        http://www.apache.org/licenses/LICENSE-2.0
// 
//    Unless required by applicable law or agreed to in writing, software
//    distributed under the License is distributed on an "AS IS" BASIS,
//    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//    See the License for the specific language governing permissions and
//    limitations under the License.

using System;
using System.Net;
using ocmengine;
using Mono.Unix;
using System.IO;
using System.Collections.Generic;

namespace ocmgtk
{


	public partial class FileDownloadProgress : Gtk.Dialog
	{		
		bool m_cancel = false;
		

		public FileDownloadProgress ()
		{
			this.Build ();
		}
		
		public void Start(List<string> files, string destFolder)
		{
			int iCount = 1;
			this.Show();
			while (Gtk.Application.EventsPending ())
						Gtk.Application.RunIteration (false);
			foreach (string url in files)
			{
				WebRequest req = WebRequest.Create(url);
				WebResponse resp = req.GetResponse();
				string fName = Utilities.GetFileShortName(url);
				if (fName == null)
				{
					//TEMP QUICK FIX: Skip this file
					iCount++;
					continue;
				}
				System.IO.FileStream fs = new FileStream(destFolder + "/" + fName, FileMode.Create);
				System.IO.Stream webstream = resp.GetResponseStream();
				fileLabel.Markup = String.Format(Catalog.GetString("File {0} of {1}"), iCount, files.Count);
				long totalKB = resp.ContentLength/8/1024;
				long totalBytes = resp.ContentLength;
				long currProg = 0;
				byte[] buff = new byte[4096];
				for(;;)
				{
					while (Gtk.Application.EventsPending ())
						Gtk.Application.RunIteration (false);
					long currKB = currProg/8/1024;
					progBar.Text = String.Format(Catalog.GetString("{0} kb of {1} kb"), currKB, totalKB);					                                                     
					progBar.Fraction = (((double) currProg / (double) totalBytes) + (double) iCount) / (double) files.Count;
					int read = webstream.Read(buff, 0, 4096);
					currProg = currProg + read;
					System.Console.WriteLine(read.ToString());
					if (read > 0 && !m_cancel)
					{
						fs.Write(buff, 0, read);
					}
					else
					{
						break;
					}
				}
				fs.Close();
				webstream.Close();
				iCount ++;
				
			}
			buttonOk.Show();
			buttonCancel.Hide();
		}
		
		protected virtual void OnOKClick (object sender, System.EventArgs e)
		{
			this.Hide();
		}
		
		protected virtual void OnCancelCLick (object sender, System.EventArgs e)
		{
			m_cancel = true;
		}
	}
}
